/*****************************************************
Chip type               : ATmega8L
AVR Core Clock frequency: 8,000000 MHz
*****************************************************/
#include <mega8.h>
#include <math.h>
#include <delay.h>

#define ticks_per_half_wave 156          // 8MHz/pwm_top/100Hz
#define ticks_per_wave (ticks_per_half_wave*2)  
#define pwm_top 0x1FF
#define sd_pin PORTB.0
#define ocp_led PORTB.4
#define battery_low_led PORTB.5
#define battery_min_voltage 155 // 11V
#define battery_sense_pin PINC.0

volatile unsigned int sin_table [];
volatile unsigned int i = 0;
volatile unsigned char timer_ticks;
volatile unsigned int U;
volatile unsigned char timer_ticks_2;
volatile bit U_good; 


void sin_init (void) 
{ 
 unsigned int n = 0; 
 unsigned int f = 0; 
 unsigned int c = 0; 
 float g= (float) PI / ticks_per_half_wave;

 do 
    { 
    if (n < ticks_per_half_wave)
    {sin_table[n]=(unsigned int)((pwm_top/2) + (sin((float) (f * g)) * ((pwm_top-15)/2))); f++;}
    else  {sin_table[n]=(unsigned int)((pwm_top/2) - (sin((float) (c * g))* ((pwm_top-15)/2))); c++;}
    
    n++; 
    } while (n<(ticks_per_wave+1)); 
 }  
//___________________________________________________________________________________
void ADC_init()
{
  ADMUX |= (1<<REFS1)|(1<<REFS0)|(1<<ADLAR)|(0<<MUX2)|(1<<MUX1)|(1<<MUX0);
  ADCSRA |= (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
}
//___________________________________________________________________________________
void port_init ()
{
DDRB = 0b00110111;
PORTB = 0b00000000;

DDRD = 0b00000000;
PORTD = 0b00000000;

DDRC = 0b00000000;
PORTC = 0b00000001;
}
//___________________________________________________________________________________
void ACMP_init () 
{ACSR = 0b01000010; } 
//___________________________________________________________________________________
void TCCR0_init ()
{
TCCR0=0x05;
TIMSK |= (1<<TOIE0);
}
//___________________________________________________________________________________
void TCCR1_init ()
{
// Clock source: System Clock
// Clock value: 8000,000 kHz
// Mode: Fast PWM top=0x01FF
// OC1A output: Non-Inv.
// OC1B output: Inverted
// Timer1 Overflow Interrupt: On

TCCR1A=0xB2; //0b10110010
TCCR1B=0x09; //0b00001001

TIMSK |= (1<<TOIE1);
}
//___________________________________________________________________________________
void ADC_read ()
{
static unsigned int tmp_U = 0;
static unsigned char i_time = 255; 
 
 if (ADCSRA.ADSC == 0)
 {
 if (i_time) {tmp_U += ADCH; i_time--;}
 else {U = (tmp_U >> 8); tmp_U = 0; i_time = 255;}
  
 ADCSRA |= (1<<ADSC);
 }
}
//___________________________________________________________________________________
void battery_check ()
{
if (U < battery_min_voltage) {sd_pin = 0; battery_low_led = 1; U_good = 0;}
else if (U > (battery_min_voltage + 15))  {battery_low_led = 0;}
{ 
if (U_good == 0) {timer_ticks_2 = 30; U_good = 1;}
if ((timer_ticks_2 == 0)&&(timer_ticks == 0)) {sd_pin = 1; battery_low_led = 0;} 
}
}
//___________________________________________________________________________________
void battery_check_2 ()
{
 if (battery_sense_pin) {sd_pin = 0; battery_low_led = 1; U_good = 0;}
 else 
 {
 battery_low_led = 0; 
 if (U_good == 0) {timer_ticks_2 = 120; U_good = 1;}
 if ((timer_ticks_2 == 0)&&(timer_ticks == 0)) {sd_pin = 1;} 
 } 
}
//___________________________________________________________________________________
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
if (timer_ticks) {timer_ticks -- ;}
if (timer_ticks_2) {timer_ticks_2 -- ;}
battery_check_2 ();
}
//___________________________________________________________________________________
interrupt [TIM1_OVF] void timer1_ovf_isr(void)
{
OCR1A =  sin_table [i]; 
OCR1B =  sin_table [i];

i++; 
if  (i > ticks_per_wave) {i=0;}
}
//___________________________________________________________________________________
void ocp ()
{
if (ACSR.ACO == 0) {timer_ticks = 60;}
if (timer_ticks) {sd_pin = 0; ocp_led = 1;}
else if ((U_good)&&(timer_ticks_2 == 0)) {sd_pin = 1; ocp_led= 0;}
else {ocp_led= 0;}
}
//___________________________________________________________________________________


void main(void)
{
port_init ();
sin_init ();
//ADC_init ();
ACMP_init ();
TCCR0_init ();
TCCR1_init ();

#asm("sei")

while (1)
{
    ocp ();
}
}
